package common

func IntToPtr(x int) *int {
	return &x
}

func Uint64ToPtr(x uint64) *uint64 {
	return &x
}

func BoolToPtr(x bool) *bool {
	return &x
}

func StringToPtr(x string) *string {
	return &x
}
